@@ -51,8 +51,7 @@
 #include <limits>
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
 
 class ImageBuf;
 
@@ -177,8 +176,12 @@ public:
     /// made, whichever comes first. If a non-NULL imagecache is supplied,
     /// it will specifiy a custom ImageCache to use; if otherwise, the
     /// global/shared ImageCache will be used.
-    ImageBuf (const std::string &name, int subimage=0, int miplevel=0,
-              ImageCache *imagecache = NULL);
+    /// If 'config' is not NULL, it points to an ImageSpec giving requests
+    /// or special instructions to be passed on to the eventual
+    /// ImageInput::open() call.
+    explicit ImageBuf (string_view name, int subimage=0, int miplevel=0,
+                       ImageCache *imagecache = NULL,
+                       const ImageSpec *config = NULL);
 
     /// Construct an ImageBuf to read the named image -- but don't actually
     /// read it yet!  The image will actually be read when other methods
@@ -186,17 +189,17 @@ public:
     /// init_spec() or read() is made, whichever comes first. If a non-NULL
     /// imagecache is supplied, it will specifiy a custom ImageCache to use;
     /// if otherwise, the global/shared ImageCache will be used.
-    ImageBuf (const std::string &name, ImageCache *imagecache);
+    ImageBuf (string_view name, ImageCache *imagecache);
 
     /// Construct an Imagebuf given a proposed spec describing the image
     /// size and type, and allocate storage for the pixels of the image
     /// (whose values will be uninitialized).
-    ImageBuf (const ImageSpec &spec);
+    explicit ImageBuf (const ImageSpec &spec);
 
     /// Construct an Imagebuf given both a name and a proposed spec
     /// describing the image size and type, and allocate storage for
     /// the pixels of the image (whose values will be undefined).
-    ImageBuf (const std::string &name, const ImageSpec &spec);
+    ImageBuf (string_view name, const ImageSpec &spec);
 
     /// Construct an ImageBuf that "wraps" a memory buffer owned by the
     /// calling application.  It can write pixels to this buffer, but
@@ -206,7 +209,7 @@ public:
     /// Construct an ImageBuf that "wraps" a memory buffer owned by the
     /// calling application.  It can write pixels to this buffer, but
     /// can't change its resolution or data type.
-    ImageBuf (const std::string &name, const ImageSpec &spec, void *buffer);
+    ImageBuf (string_view name, const ImageSpec &spec, void *buffer);
 
     /// Construct a copy of an ImageBuf.
     ///
@@ -229,12 +232,16 @@ public:
 
     /// Forget all previous info, reset this ImageBuf to a new image
     /// that is uninitialized (no pixel values, no size or spec).
-    void reset (const std::string &name, int subimage, int miplevel,
-                ImageCache *imagecache = NULL);
+    /// If 'config' is not NULL, it points to an ImageSpec giving requests
+    /// or special instructions to be passed on to the eventual
+    /// ImageInput::open() call.
+    void reset (string_view name, int subimage, int miplevel,
+                ImageCache *imagecache = NULL,
+                const ImageSpec *config = NULL);
 
     /// Forget all previous info, reset this ImageBuf to a new image
     /// that is uninitialized (no pixel values, no size or spec).
-    void reset (const std::string &name, ImageCache *imagecache=NULL);
+    void reset (string_view name, ImageCache *imagecache=NULL);
 
     /// Forget all previous info, reset this ImageBuf to a blank
     /// image of the given dimensions.
@@ -242,16 +249,7 @@ public:
 
     /// Forget all previous info, reset this ImageBuf to a blank
     /// image of the given name and dimensions.
-    void reset (const std::string &name, const ImageSpec &spec);
-
-    /// Copy spec to *this, and then allocate enough space the right
-    /// size for an image described by the format spec.  If the ImageBuf
-    /// already has allocated pixels, their values will not be preserved
-    /// if the new spec does not describe an image of the same size and
-    /// data type as it used to be.
-    /// DEPRECATED (1.3) -- I don't see any reason why this should be
-    /// favored over reset(spec).
-    void alloc (const ImageSpec &spec);
+    void reset (string_view name, const ImageSpec &spec);
 
     /// Which type of storage is being used for the pixels?
     IBStorage storage () const;
@@ -273,14 +271,14 @@ public:
     /// header to fill out the spec correctly.  Return true if this
     /// succeeded, false if the file could not be read.  But don't
     /// allocate or read the pixels.
-    bool init_spec (const std::string &filename,
+    bool init_spec (string_view filename,
                     int subimage, int miplevel);
 
     /// Write the image to the named file and file format ("" means to infer
     /// the type from the filename extension). Return true if all went ok,
     /// false if there were errors writing.
-    bool write (const std::string &filename,
-                const std::string &fileformat = std::string(),
+    bool write (string_view filename,
+                string_view fileformat = string_view(),
                 ProgressCallback progress_callback=NULL,
                 void *progress_callback_data=NULL) const;
 
@@ -292,13 +290,6 @@ public:
     /// any subsequent write().
     void set_write_tiles (int width=0, int height=0, int depth=0);
 
-    /// DEPRECATED (1.3) synonym for write().  Kept for now for backward
-    /// compatibility.
-    bool save (const std::string &filename = std::string(),
-               const std::string &fileformat = std::string(),
-               ProgressCallback progress_callback=NULL,
-               void *progress_callback_data=NULL) const;
-
     /// Write the image to the open ImageOutput 'out'.  Return true if
     /// all went ok, false if there were errors writing.  It does NOT
     /// close the file when it's done (and so may be called in a loop to
@@ -307,6 +298,18 @@ public:
                 ProgressCallback progress_callback=NULL,
                 void *progress_callback_data=NULL) const;
 
+    /// Force the ImageBuf to be writeable. That means that if it was
+    /// previously backed by ImageCache (storage was IMAGECACHE), it will
+    /// force a full read so that the whole image is in local memory. This
+    /// will invalidate any current iterators on the image. It has no effect
+    /// if the image storage not IMAGECACHE.  Return true if it works
+    /// (including if no read was necessary), false if something went
+    /// horribly wrong. If keep_cache_type is true, it preserves any IC-
+    /// forced data types (you might want to do this if it is critical that
+    /// the apparent data type doesn't change, for example if you are
+    /// calling make_writeable from within a type-specialized function).
+    bool make_writeable (bool keep_cache_type = false);
+
     /// Copy all the metadata from src to *this (except for pixel data
     /// resolution, channel information, and data format).
     void copy_metadata (const ImageBuf &src);
@@ -334,6 +337,9 @@ public:
     /// converted automatically to the data type of the app buffer.
     bool copy (const ImageBuf &src);
 
+    /// copy(src), but with optional override of pixel data type
+    bool copy (const ImageBuf &src, TypeDesc format /*= TypeDesc::UNKNOWN*/);
+
     /// Swap with another ImageBuf
     void swap (ImageBuf &other) { std::swap (m_impl, other.m_impl); }
 
@@ -371,12 +377,12 @@ public:
 
     /// Return the name of this image.
     ///
-    const std::string & name (void) const;
+    string_view name (void) const;
 
     /// Return the name of the image file format of the disk file we
     /// read into this image.  Returns an empty string if this image
     /// was not the result of a read().
-    const std::string & file_format_name (void) const;
+    string_view file_format_name (void) const;
 
     /// Return the index of the subimage are we currently viewing
     ///
@@ -407,40 +413,53 @@ public:
     ///
     float getchannel (int x, int y, int z, int c, WrapMode wrap=WrapBlack) const;
 
-    /// Retrieve the pixel value by x and y coordintes (on [0,res-1]),
+    /// Retrieve the pixel value by x and y pixel indices,
     /// storing the floating point version in pixel[].  Retrieve at most
     /// maxchannels (will be clamped to the actual number of channels).
     void getpixel (int x, int y, float *pixel, int maxchannels=1000) const {
         getpixel (x, y, 0, pixel, maxchannels);
     }
 
-    /// Retrieve the pixel value by x, y, z coordintes (on [0,res-1]),
+    /// Retrieve the pixel value by x, y, z pixel indices,
     /// storing the floating point version in pixel[].  Retrieve at most
     /// maxchannels (will be clamped to the actual number of channels).
     void getpixel (int x, int y, int z, float *pixel, int maxchannels=1000,
                    WrapMode wrap=WrapBlack) const;
 
-    /// Linearly interpolate at pixel coordinates (x,y), where (0,0) is
-    /// the upper left corner, (xres,yres) the lower right corner of
-    /// the pixel data.
+    /// Sample the image plane at coordinates (x,y), using linear
+    /// interpolation between pixels, placing the result in pixel[0..n-1]
+    /// where n is the smaller of maxchannels or the actual number of
+    /// channels stored in the buffer.  It is up to the application to
+    /// ensure that pixel points to enough memory to hold the required
+    /// number of channels. Note that pixel data values themselves are at
+    /// the pixel centers, so pixel (i,j) is at image plane coordinate
+    /// (i+0.5, j+0.5).
     void interppixel (float x, float y, float *pixel,
                       WrapMode wrap=WrapBlack) const;
 
-    /// Linearly interpolate at image data NDC coordinates (s,t), where
-    /// (0,0) is the upper left corner of the pixel data window, (1,1)
-    /// the lower right corner of the pixel data.
-    /// FIXME -- lg thinks that this is stupid, and the only useful NDC
-    /// space is the one used by interppixel_NDC_full.  We should deprecate
-    /// this in the future.
+    /// Linearly interpolate at NDC coordinates (s,t), where (0,0) is
+    /// the upper left corner of the display window, (1,1) the lower
+    /// right corner of the display window.
     void interppixel_NDC (float s, float t, float *pixel,
                           WrapMode wrap=WrapBlack) const;
 
-    /// Linearly interpolate at space coordinates (s,t), where (0,0) is
-    /// the upper left corner of the display window, (1,1) the lower
-    /// right corner of the display window.
+    /// DEPCRECATED (1.5) synonym for interppixel_NDC.
     void interppixel_NDC_full (float s, float t, float *pixel,
                                WrapMode wrap=WrapBlack) const;
 
+    /// Bicubic interpolation at pixel coordinates (x,y), where (0,0) is
+    /// the upper left corner, (xres,yres) the lower right corner of
+    /// the pixel data.
+    void interppixel_bicubic (float x, float y, float *pixel,
+                              WrapMode wrap=WrapBlack) const;
+
+    /// Bicubic interpolattion at NDC space coordinates (s,t), where (0,0)
+    /// is the upper left corner of the display (aka "full") window, (1,1)
+    /// the lower right corner of the display window.
+    void interppixel_bicubic_NDC (float s, float t, float *pixel,
+                                  WrapMode wrap=WrapBlack) const;
+
+
     /// Set the pixel with coordinates (x,y,0) to have the values in
     /// pixel[0..n-1].  The number of channels copied, n, is the minimum
     /// of maxchannels and the actual number of channels in the image.
@@ -459,17 +478,32 @@ public:
     /// maxchannels (will be clamped to the actual number of channels).
     void setpixel (int i, const float *pixel, int maxchannels=1000);
 
-    /// Retrieve the rectangle of pixels spanning [xbegin..xend) X
-    /// [ybegin..yend) X [zbegin..zend), channels [chbegin,chend) (all
-    /// with exclusive 'end'), specified as integer pixel coordinates,
-    /// at the current subimage and MIP-map level, storing the pixel
-    /// values beginning at the address specified by result and with the
-    /// given strides (by default, AutoStride means the usual contiguous
-    /// packing of pixels) and converting into the data type described
-    /// by 'format'.  It is up to the caller to ensure that result
-    /// points to an area of memory big enough to accommodate the
-    /// requested rectangle.  Return true if the operation could be
-    /// completed, otherwise return false.
+    /// Retrieve the rectangle of pixels spanning the ROI (including
+    /// channels) at the current subimage and MIP-map level, storing the
+    /// pixel values beginning at the address specified by result and with
+    /// the given strides (by default, AutoStride means the usual contiguous
+    /// packing of pixels) and converting into the data type described by
+    /// 'format'.  It is up to the caller to ensure that result points to an
+    /// area of memory big enough to accommodate the requested rectangle.
+    /// Return true if the operation could be completed, otherwise return
+    /// false.
+    bool get_pixels (ROI roi, TypeDesc format, void *result,
+                     stride_t xstride=AutoStride,
+                     stride_t ystride=AutoStride,
+                     stride_t zstride=AutoStride) const;
+
+    /// Copy the data into the given ROI of the ImageBuf. The data points to
+    /// values specified by 'format', with layout detailed by the stride
+    /// values (in bytes, with AutoStride indicating "contiguous" layout).
+    /// It is up to the caller to ensure that data points to an area of
+    /// memory big enough to account for the ROI. Return true if the
+    /// operation could be completed, otherwise return false.
+    bool set_pixels (ROI roi, TypeDesc format, const void *data,
+                     stride_t xstride=AutoStride,
+                     stride_t ystride=AutoStride,
+                     stride_t zstride=AutoStride);
+
+    /// DEPRECATED (1.6) -- use get_pixels(ROI, ...) instead.
     bool get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                              int zbegin, int zend, int chbegin, int chend,
                              TypeDesc format, void *result,
@@ -477,16 +511,7 @@ public:
                              stride_t ystride=AutoStride,
                              stride_t zstride=AutoStride) const;
 
-    /// Retrieve the rectangle of pixels spanning [xbegin..xend) X
-    /// [ybegin..yend) X [zbegin..zend) (with exclusive 'end'),
-    /// specified as integer pixel coordinates, at the current MIP-map
-    /// level, storing the pixel values beginning at the address
-    /// specified by result and with the given strides (by default,
-    /// AutoStride means the usual contiguous packing of pixels) and
-    /// converting into the data type described by 'format'.  It is up
-    /// to the caller to ensure that result points to an area of memory
-    /// big enough to accommodate the requested rectangle.  Return true
-    /// if the operation could be completed, otherwise return false.
+    /// DEPRECATED (1.6) -- use get_pixels(ROI, ...) instead.
     bool get_pixels (int xbegin, int xend, int ybegin, int yend,
                      int zbegin, int zend,
                      TypeDesc format, void *result,
@@ -494,16 +519,7 @@ public:
                      stride_t ystride=AutoStride,
                      stride_t zstride=AutoStride) const;
 
-    /// Retrieve the rectangle of pixels spanning [xbegin..xend) X
-    /// [ybegin..yend) (with exclusive 'end'), specified as integer
-    /// pixel coordinates, at the current MIP-map level, storing the
-    /// pixel values beginning at the address specified by result and
-    /// with the given strides (by default, AutoStride means the usual
-    /// contiguous packing of pixels), converting to the type <T> in the
-    /// process.  It is up to the caller to ensure that result points to
-    /// an area of memory big enough to accommodate the requested
-    /// rectangle.  Return true if the operation could be completed,
-    /// otherwise return false.
+    /// DEPRECATED (1.6) -- use get_pixels(ROI, ...) instead.
     template<typename T>
     bool get_pixel_channels (int xbegin, int xend, int ybegin, int yend,
                              int zbegin, int zend, int chbegin, int chend,
@@ -511,6 +527,7 @@ public:
                              stride_t ystride=AutoStride,
                              stride_t zstride=AutoStride) const;
 
+    /// DEPRECATED (1.6) -- use get_pixels(ROI, ...) instead.
     template<typename T>
     bool get_pixels (int xbegin, int xend, int ybegin, int yend,
                      int zbegin, int zend, T *result,
@@ -522,12 +539,7 @@ public:
                                    xstride, ystride, zstride);
     }
 
-    /// Even safer version of get_pixels: Retrieve the rectangle of
-    /// pixels spanning [xbegin..xend) X [ybegin..yend) (with exclusive
-    /// 'end'), specified as integer pixel coordinates, at the current
-    /// MIP-map level, storing the pixel values in the 'result' vector
-    /// (even allocating the right size).  Return true if the operation
-    /// could be completed, otherwise return false.
+    /// DEPRECATED (1.6) -- use get_pixels(ROI, ...) instead.
     template<typename T>
     bool get_pixels (int xbegin_, int xend_, int ybegin_, int yend_,
                       int zbegin_, int zend_,
@@ -538,7 +550,9 @@ public:
                            &result[0]);
     }
 
+
     int orientation () const;
+    void set_orientation (int orient);
 
     int oriented_width () const;
     int oriented_height () const;
@@ -590,14 +604,6 @@ public:
     void set_full (int xbegin, int xend, int ybegin, int yend,
                    int zbegin, int zend);
 
-    /// Set the "full" (a.k.a. display) window to [xbegin,xend) x
-    /// [ybegin,yend) x [zbegin,zend).  If bordercolor is not NULL, also
-    /// set the spec's "oiio:bordercolor" attribute.
-    /// DEPRECATED (1.3) -- we don't use the 'bordercolor' parameter, and
-    /// it seems strange, so let's phase it out.
-    void set_full (int xbegin, int xend, int ybegin, int yend,
-                   int zbegin, int zend, const float *bordercolor);
-
     /// Return pixel data window for this ImageBuf as a ROI.
     ROI roi () const;
 
@@ -659,10 +665,34 @@ public:
     /// deep samples.
     float deep_value (int x, int y, int z, int c, int s) const;
 
+    /// Retrieve deep sample value within a pixel, as an untigned int.
+    uint32_t deep_value_uint (int x, int y, int z, int c, int s) const;
+
+    /// Set the number of deep samples for a particular pixel.
+    void set_deep_samples (int x, int y, int z, int nsamples);
+
+    /// Set deep sample value within a pixel, as a float.
+    void set_deep_value (int x, int y, int z, int c, int s, float value);
+    /// Set deep sample value within a pixel, as a uint32.
+    void set_deep_value_uint (int x, int y, int z, int c, int s, uint32_t value);
+
+    /// Allocate all the deep samples, called after deepdata()->nsamples
+    /// is set.
+    void deep_alloc ();
+
     /// Retrieve the "deep" data.
     DeepData *deepdata ();
     const DeepData *deepdata () const;
 
+    /// Set the current thread-spawning policy: the maximum number of
+    /// threads that may be spawned by ImagBuf internals. A value of 1
+    /// means all work will be done by the calling thread; 0 means to use
+    /// the global OIIO::attribute("threads") value.
+    void threads (int n) const;
+
+    /// Retrieve the current thread-spawning policy of this ImageBuf.
+    int threads () const;
+
     friend class IteratorBase;
 
     class IteratorBase {
@@ -785,7 +815,7 @@ public:
         }
 
         /// Retrieve the number of deep data samples at this pixel.
-        int deep_samples () { return m_ib->deep_samples (m_x, m_y, m_z); }
+        int deep_samples () const { return m_ib->deep_samples (m_x, m_y, m_z); }
 
         /// Return the wrap mode
         WrapMode wrap () const { return m_wrap; }
@@ -974,6 +1004,25 @@ public:
                 }
             }
         }
+
+        // Set to the "done" position
+        void pos_done () {
+            m_valid = false;
+            m_x = m_rng_xbegin;
+            m_y = m_rng_ybegin;
+            m_z = m_rng_zend;
+        }
+
+        // Make sure it's writeable. Use with caution!
+        void make_writeable () {
+            if (! m_localpixels) {
+                const_cast<ImageBuf*>(m_ib)->make_writeable (true);
+                DASSERT (m_ib->storage() != IMAGECACHE);
+                m_tile = NULL;
+                m_proxydata = NULL;
+                init_ib (m_wrap);
+            }
+        }
     };
 
     /// Templated class for referring to an individual pixel in an
@@ -1002,7 +1051,11 @@ public:
         Iterator (ImageBuf &ib, WrapMode wrap=WrapDefault)
             : IteratorBase(ib,wrap)
         {
+            make_writeable ();
             pos (m_rng_xbegin,m_rng_ybegin,m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Construct from an ImageBuf and a specific pixel index.
         /// The iteration range is the full image.
@@ -1010,13 +1063,18 @@ public:
                   WrapMode wrap=WrapDefault)
             : IteratorBase(ib,wrap)
         {
+            make_writeable ();
             pos (x, y, z);
         }
         /// Construct read-write iteration region from ImageBuf and ROI.
         Iterator (ImageBuf &ib, const ROI &roi, WrapMode wrap=WrapDefault)
             : IteratorBase (ib, roi, wrap)
         {
+            make_writeable ();
             pos (m_rng_xbegin, m_rng_ybegin, m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Construct from an ImageBuf and designated region -- iterate
         /// over region, starting with the upper left pixel.
@@ -1025,13 +1083,18 @@ public:
                   WrapMode wrap=WrapDefault)
             : IteratorBase(ib, xbegin, xend, ybegin, yend, zbegin, zend, wrap)
         {
+            make_writeable ();
             pos (m_rng_xbegin, m_rng_ybegin, m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Copy constructor.
         ///
         Iterator (Iterator &i)
             : IteratorBase (i.m_ib, i.m_wrap)
         {
+            make_writeable ();
             pos (i.m_x, i.m_y, i.m_z);
         }
 
@@ -1068,10 +1131,26 @@ public:
 
         void * rawptr () const { return m_proxydata; }
 
+        /// Set the number of deep data samples at this pixel. (Only use
+        /// this if deep_alloc() has not yet been called on the buffer.)
+        void set_deep_samples (int n) {
+            return const_cast<ImageBuf*>(m_ib)->set_deep_samples (m_x, m_y, m_z, n);
+        }
+
         /// Retrieve the deep data value of sample s of channel c.
         USERT deep_value (int c, int s) const {
             return convert_type<float,USERT>(m_ib->deep_value (m_x, m_y, m_z, c, s));
         }
+
+        /// Set the deep data value of sample s of channel c. (Only use this
+        /// if deep_alloc() has been called.)
+        void set_deep_value (int c, int s, float value) {
+            return const_cast<ImageBuf*>(m_ib)->set_deep_value (m_x, m_y, m_z, c, s, value);
+        }
+        void set_deep_value (int c, int s, uint32_t value) {
+            return const_cast<ImageBuf*>(m_ib)->set_deep_value_uint (m_x, m_y, m_z, c, s, value);
+        }
+
     };
 
 
@@ -1086,6 +1165,9 @@ public:
             : IteratorBase(ib,wrap)
         {
             pos (m_rng_xbegin,m_rng_ybegin,m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Construct from an ImageBuf and a specific pixel index.
         /// The iteration range is the full image.
@@ -1101,6 +1183,9 @@ public:
             : IteratorBase (ib, roi, wrap)
         {
             pos (m_rng_xbegin, m_rng_ybegin, m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Construct from an ImageBuf and designated region -- iterate
         /// over region, starting with the upper left pixel.
@@ -1110,6 +1195,9 @@ public:
             : IteratorBase(ib, xbegin, xend, ybegin, yend, zbegin, zend, wrap)
         {
             pos (m_rng_xbegin, m_rng_ybegin, m_rng_zbegin);
+            if (m_rng_xbegin == m_rng_xend || m_rng_ybegin == m_rng_yend
+                  || m_rng_zbegin == m_rng_zend)
+                pos_done();  // make empty range look "done"
         }
         /// Copy constructor.
         ///
@@ -1157,12 +1245,6 @@ protected:
     ImageBufImpl * impl () { return m_impl; }
     const ImageBufImpl * impl () const { return m_impl; }
 
-    // Copy src's pixels into *this.  Pixels must already be local
-    // (either owned or wrapped) and the resolution and number of
-    // channels must match src.  Data type is allowed to be different,
-    // however, with automatic conversion upon copy.
-    void copy_from (const ImageBuf &src);
-
     // Reset the ImageCache::Tile * to reserve and point to the correct
     // tile for the given pixel, and return the ptr to the actual pixel
     // within the tile.
@@ -1189,7 +1271,6 @@ protected:
 };
 
 
-}
-OIIO_NAMESPACE_EXIT
+OIIO_NAMESPACE_END
 
 #endif // OPENIMAGEIO_IMAGEBUF_H
